Parameters — AsyncClick Documentation (8.x)

# Parameters

Click supports two types of parameters for scripts: options and arguments. There is generally some confusion among authors of command line scripts of when to use which, so here is a quick overview of the differences. As its name indicates, an option is optional. While arguments can be optional within reason, they are much more restricted in how optional they can be.

To help you decide between options and arguments, the recommendation is to use arguments exclusively for things like going to subcommands or input filenames / URLs, and have everything else be an option instead.

## Differences

Arguments can do less than options. The following features are only available for options:

- automatic prompting for missing input

- act as flags (boolean or otherwise)

- option values can be pulled from environment variables, arguments can not

- options are fully documented in the help page, arguments are not (this is intentional as arguments might be too specific to be automatically documented)

On the other hand arguments, unlike options, can accept an arbitrary number of arguments. Options can strictly ever only accept a fixed number of arguments (defaults to 1), or they may be specified multiple times using Multiple Options.

## Parameter Types

Parameters can be of different types. Types can be implemented with different behavior and some are supported out of the box:

`str` / `click.STRING`:

  The default parameter type which indicates unicode strings.

`int` / `click.INT`:

  A parameter that only accepts integers.

`float` / `click.FLOAT`:

  A parameter that only accepts floating point values.

`bool` / `click.BOOL`:

  A parameter that accepts boolean values. This is automatically used for boolean flags. The string values “1”, “true”, “t”, “yes”, “y”, and “on” convert to `True`. “0”, “false”, “f”, “no”, “n”, and “off” convert to `False`.

`click.UUID`:

  A parameter that accepts UUID values. This is not automatically guessed but represented as `uuid.UUID`.

class `asyncclick.` `File`(mode='r', encoding=None, errors='strict', lazy=None, atomic=False)

  Declares a parameter to be a file for reading or writing. The file is automatically closed once the context tears down (after the command finished working).

  Files can be opened for reading or writing. The special value `-` indicates stdin or stdout depending on the mode.

  By default, the file is opened for reading text data, but it can also be opened in binary mode or for writing. The encoding parameter can be used to force a specific encoding.

  The lazy flag controls if the file should be opened immediately or upon first IO. The default is to be non-lazy for standard input and output streams as well as files opened for reading, lazy otherwise. When opening a file lazily for reading, it is still opened temporarily for validation, but will not be held open until first IO. lazy is mainly useful when opening for writing to avoid creating the file until it is needed.

  Starting with Click 2.0, files can also be opened atomically in which case all writes go into a separate file in the same folder and upon completion the file will be moved over to the original location. This is useful if a file regularly read by other users is modified.

  See File Arguments for more information.

class `asyncclick.` `Path`(exists=False, file_okay=True, dir_okay=True, writable=False, readable=True, resolve_path=False, allow_dash=False, path_type=None)

  The path type is similar to the `File` type but it performs different checks. First of all, instead of returning an open file handle it returns just the filename. Secondly, it can perform various basic checks about what the file or directory should be.

  Changelog

  Changed in version 6.0: allow_dash was added.

  Parameters

    - exists – if set to true, the file or directory needs to exist for this value to be valid. If this is not required and a file does indeed not exist, then all further checks are silently skipped.

    - file_okay – controls if a file is a possible value.

    - dir_okay – controls if a directory is a possible value.

    - writable – if true, a writable check is performed.

    - readable – if true, a readable check is performed.

    - resolve_path – if this is true, then the path is fully resolved before the value is passed onwards. This means that it’s absolute and symlinks are resolved. It will not expand a tilde-prefix, as this is supposed to be done by the shell only.

    - allow_dash – If this is set to True, a single dash to indicate standard streams is permitted.

    - path_type – optionally a string type that should be used to represent the path. The default is None which means the return value will be either bytes or unicode depending on what makes most sense given the input data Click deals with.

class `asyncclick.` `Choice`(choices, case_sensitive=True)

  The choice type allows a value to be checked against a fixed set of supported values. All of these values have to be strings.

  You should only pass a list or tuple of choices. Other iterables (like generators) may lead to surprising results.

  The resulting value will always be one of the originally passed choices regardless of `case_sensitive` or any `ctx.token_normalize_func` being specified.

  See Choice Options for an example.

  Parameters

    case_sensitive – Set to false to make choices case insensitive. Defaults to true.

class `asyncclick.` `IntRange`(min=None, max=None, min_open=False, max_open=False, clamp=False)

  Restrict an `click.INT` value to a range of accepted values. See Range Options.

  If `min` or `max` are not passed, any value is accepted in that direction. If `min_open` or `max_open` are enabled, the corresponding boundary is not included in the range.

  If `clamp` is enabled, a value outside the range is clamped to the boundary instead of failing.

  Changed in version 8.0: Added the `min_open` and `max_open` parameters.

class `asyncclick.` `FloatRange`(min=None, max=None, min_open=False, max_open=False, clamp=False)

  Restrict a `click.FLOAT` value to a range of accepted values. See Range Options.

  If `min` or `max` are not passed, any value is accepted in that direction. If `min_open` or `max_open` are enabled, the corresponding boundary is not included in the range.

  If `clamp` is enabled, a value outside the range is clamped to the boundary instead of failing. This is not supported if either boundary is marked `open`.

  Changed in version 8.0: Added the `min_open` and `max_open` parameters.

class `asyncclick.` `DateTime`(formats=None)

  The DateTime type converts date strings into datetime objects.

  The format strings which are checked are configurable, but default to some common (non-timezone aware) ISO 8601 formats.

  When specifying DateTime formats, you should only pass a list or a tuple. Other iterables, like generators, may lead to surprising results.

  The format strings are processed using `datetime.strptime`, and this consequently defines the format strings which are allowed.

  Parsing is tried using each format, in order, and the first format which parses successfully is used.

  Parameters

    formats – A list or tuple of date format strings, in the order in which they should be tried. Defaults to `'%Y-%m-%d'`, `'%Y-%m-%dT%H:%M:%S'`, `'%Y-%m-%d %H:%M:%S'`.

Custom parameter types can be implemented by subclassing `click.ParamType`. For simple cases, passing a Python function that fails with a ValueError is also supported, though discouraged.

## Parameter Names

Parameters (both options and arguments) have a name that will be used as the Python argument name when calling the decorated function with values.

Arguments take only one positional name. To provide a different name for use in help text, see Truncating Help Texts.

Options can have many names that may be prefixed with one or two dashes. Names with one dash are parsed as short options, names with two are parsed as long options. If a name is not prefixed, it is used as the Python argument name and not parsed as an option name. Otherwise, the first name with a two dash prefix is used, or the first with a one dash prefix if there are none with two. The prefix is removed and dashes are converted to underscores to get the Python argument name.

## Implementing Custom Types

To implement a custom type, you need to subclass the `ParamType` class. Override the `convert()` method to convert the value from a string to the correct type.

The following code implements an integer type that accepts hex and octal numbers in addition to normal integers, and converts them into regular integers.

    import asyncclick as click

    class BasedIntParamType(click.ParamType):
        name = "integer"

        def convert(self, value, param, ctx):
            try:
                if value[:2].lower() == "0x":
                    return int(value[2:], 16)
                elif value[:1] == "0":
                    return int(value, 8)
                return int(value, 10)
            except TypeError:
                self.fail(
                    "expected string for int() conversion, got "
                    f"{value!r} of type {type(value).__name__}",
                    param,
                    ctx,
                )
            except ValueError:
                self.fail(f"{value!r} is not a valid integer", param, ctx)

    BASED_INT = BasedIntParamType()

The `name` attribute is optional and is used for documentation. Call `fail()` if conversion fails. The `param` and `ctx` arguments may be `None` in some cases such as prompts.